from subprocess import call, PIPE
from django.core.management.base import BaseCommand
from image_diet import settings


class Command(BaseCommand):
    help = ("Check which external image diet tools are "
            "available and suggest settings.py configuration.")

    def handle(self, *args, **options):
        tools = (
            'jpegoptim',
            'jpegtran',
            'gifsicle',
            'optipng',
            'advpng',
            'pngcrush',
            'pngquant',
        )
        not_found = []
        for tool in tools:
            setting_name = "DIET_" + tool.upper()
            if getattr(settings, setting_name):
                retcode = call("which %s" % tool, shell=True, stdout=PIPE)
                if retcode == 0:
                    self.stdout.write('Found: %s\n' % tool)
                else:
                    self.stdout.write('MISSING: %s\n' % tool)
                    not_found.append(tool)
            else:  # Tool turned off
                self.stdout.write('Disabled: %s\n' % tool)
        if len(not_found):
            off_settings = ["DIET_%s = False" % tool.upper() for tool
                            in not_found]
            self.stdout.write("\n")
            self.stdout.write("You can disable missing tools by adding following lines to your settings.py: \n")
            self.stdout.write("\n".join(off_settings))
            self.stdout.write("\n")



import os
from image_diet import settings
from django.conf import settings as main_settings
from django.contrib.staticfiles.storage import StaticFilesStorage


class ImageDietFileSystemStorage(StaticFilesStorage):

    def post_process(self, files, *args, **kwargs):
        results = []
        if settings.DIET_COMPRESS_STATIC_IMAGES:
            if 'image_diet' not in main_settings.INSTALLED_APPS:
                raise NotImplementedError("You need to install django_image_diet to use DIET_COMPRESS_STATIC_IMAGES")

            from image_diet.diet import squeeze
            for f in files:
                processed_file = squeeze(os.path.join(main_settings.STATIC_ROOT, f))
                results.append([f, processed_file, True if processed_file is not None else False])
        return results



import logging
import time
import datetime
import os
from subprocess import call, PIPE
from imghdr import what as determinetype
from image_diet import settings

logger = logging.getLogger('image_diet')


def squeeze_jpeg():
    ''' Prefer jpegtran to jpegoptim since it makes smaller images
    and can create progressive jpegs (smaller and faster to load)'''
    if not settings.DIET_JPEGTRAN and not settings.DIET_JPEGOPTIM:  # Can't do anything
        return ""
    if not settings.DIET_JPEGTRAN:
        return u"jpegoptim -f --strip-all '%(file)s'"
    return (u"jpegtran -copy none -progressive -optimize -outfile '%(file)s'.diet '%(file)s' "
            "&& mv '%(file)s.diet' '%(file)s'")


def squeeze_gif():
    '''Gifsicle only optimizes animations.

    Eventually add support to change gifs to png8.'''
    return (u"gifsicle -O2 '%(file)s' > '%(file)s'.diet "
            "&& mv '%(file)s.diet' '%(file)s'") if settings.DIET_GIFSICLE else ""


def squeeze_png():
    commands = []
    if settings.DIET_OPTIPNG:
        commands.append(u"optipng -force -o7 '%(file)s'")
    if settings.DIET_ADVPNG:
        commands.append(u"advpng -z4 '%(file)s'")
    if settings.DIET_PNGCRUSH:
        commands.append(
            (u"pngcrush -rem gAMA -rem alla -rem cHRM -rem iCCP -rem sRGB "
             u"-rem time '%(file)s' '%(file)s.diet' "
             u"&& mv '%(file)s.diet' '%(file)s'")
        )
    if settings.DIET_PNGQUANT and not settings.DIET_PNGQUANT_BRUTE:
        commands.append(u"pngquant --force --verbose --quality 90-100 '%(file)s' -o '%(file)s' ")
    if settings.DIET_PNGQUANT_BRUTE:
        commands.append(u"pngquant --force --verbose '%(file)s' -o '%(file)s' ")
    return " && ".join(commands)


def backup_copy(path):
    if settings.DIET_IMAGE_BACKUPS:
        timestamp = time.time()
        timestamp = datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d-%H:%M:%S')
        call("cp '%(file)s' '%(file)s-%(timestamp)s'" % {'file': path, 'timestamp': timestamp},
             shell=True, stdout=PIPE)


def get_flag_path(path):
    filename = os.path.basename(path)
    directory = os.path.split(path)[0]
    flagged_file_name = '.%s.%s' % (filename, settings.DIET_COMPRESSED_FLAG)
    flag_path = os.path.join(directory, flagged_file_name)
    return flag_path


def create_processed_file_flag(path):
    flag_path = get_flag_path(path)
    call("touch '%s'" % flag_path, shell=True)


def squeeze(path, backup=settings.DIET_IMAGE_BACKUPS, flag_processed_file=settings.DIET_FLAG_PROCESSED_FILE,
            new_only=settings.DIET_NEW_FILES_ONLY):
    '''Returns path of optimized image or None if something went wrong.'''
    if not os.path.exists(path):
        logger.error("'%s' does not point to a file." % path)
        return None

    filetype = determinetype(path)

    if backup:
        backup_copy(path)

    squeeze_cmd = ""
    if filetype == "jpeg":
        squeeze_cmd = squeeze_jpeg()
    elif filetype == "gif":
        squeeze_cmd = squeeze_gif()
    elif filetype == "png":
        squeeze_cmd = squeeze_png()

    if new_only:
        should_process_file = False
        flag_path = get_flag_path(path)
        flag_path = get_flag_path(path)

        if not os.path.exists(flag_path):
            should_process_file = True
        else:
            file_mt = os.path.getmtime(path)
            flag_mt = os.path.getmtime(flag_path)

            if flag_mt < file_mt:
                should_process_file = True

        if not should_process_file:
            squeeze_cmd = False

    if squeeze_cmd:
        try:
            retcode = call(squeeze_cmd % {'file': path},
                           shell=True, stdout=PIPE)
            if flag_processed_file:
                create_processed_file_flag(path)
        except:
            raise
            logger.error('Squeezing failed with parameters:')
            logger.error(squeeze_cmd[filetype] % {'file': path})
            logger.exception()
            return None

        if retcode != 0:
            # Failed.
            logger.error(
                ('Squeezing failed. '
                 'Likely because you are missing one of required utilities.'))
            return None
    return path



import os
from os.path import join, dirname
from shutil import copyfile
from subprocess import call, PIPE
from django.test import TestCase
from image_diet import diet

TEST_DIR = join(dirname(__file__), 'test_files')


class DietTest(TestCase):

    def setUp(self):
        image_path = join(TEST_DIR, 'stockholm.jpg')

        self.test_image_path = join(TEST_DIR, 'test_image.jpg')
        copyfile(image_path, self.test_image_path)

    def tearDown(self):
        os.remove(self.test_image_path)

    def have_jpeg_tools(self):
        retcode = call("which jpegoptim || which jpegtran", shell=True,
                       stdout=PIPE)
        return True if retcode is 0 else False

    def test_squeeze_jpeg(self):
        diet.settings.DIET_JPEGOPTIM = False
        diet.settings.DIET_JPEGTRAN = False
        self.assertEqual(diet.squeeze_jpeg(), "")

        diet.settings.DIET_JPEGTRAN = True
        self.assertEqual(
            diet.squeeze_jpeg(),
            (u"jpegtran -copy none -progressive -optimize -outfile '%(file)s'.diet '%(file)s' "
                "&& mv '%(file)s.diet' '%(file)s'")
        )

        diet.settings.DIET_JPEGOPTIM = True
        diet.settings.DIET_JPEGTRAN = False
        self.assertEqual(diet.squeeze_jpeg(), u"jpegoptim -f --strip-all '%(file)s'")

    def test_squeeze_png(self):
        diet.settings.DIET_OPTIPNG = False
        diet.settings.DIET_ADVPNG = False
        diet.settings.DIET_PNGCRUSH = False
        self.assertEqual(diet.squeeze_png(), "")

        diet.settings.DIET_OPTIPNG = True
        diet.settings.DIET_ADVPNG = False
        diet.settings.DIET_PNGCRUSH = False
        self.assertEqual(diet.squeeze_png(), u"optipng -force -o7 '%(file)s'")

        diet.settings.DIET_OPTIPNG = False
        diet.settings.DIET_ADVPNG = True
        diet.settings.DIET_PNGCRUSH = False
        self.assertEqual(diet.squeeze_png(), u"advpng -z4 '%(file)s'")

        diet.settings.DIET_OPTIPNG = False
        diet.settings.DIET_ADVPNG = False
        diet.settings.DIET_PNGCRUSH = True
        self.assertEqual(
            diet.squeeze_png(),
            (u"pngcrush -rem gAMA -rem alla -rem cHRM -rem iCCP -rem sRGB "
             u"-rem time '%(file)s' '%(file)s.diet' "
             u"&& mv '%(file)s.diet' '%(file)s'")
        )

        diet.settings.DIET_OPTIPNG = True
        diet.settings.DIET_ADVPNG = True
        diet.settings.DIET_PNGCRUSH = False
        self.assertEqual(diet.squeeze_png(), u"optipng -force -o7 '%(file)s' && advpng -z4 '%(file)s'")

    def test_squeeze_gif(self):
        diet.settings.DIET_GIFSICLE = True
        self.assertEqual(
            diet.squeeze_gif(),
            (u"gifsicle -O2 b'%(file)s' > '%(file)s'.diet "
                "&& mv '%(file)s.diet' '%(file)s'")
        )

        diet.settings.DIET_GIFSICLE = False
        self.assertEqual(diet.squeeze_gif(), "")

    def test_squeeze(self):
        self.assertEqual(diet.squeeze('/tmp/bla'), None)

        old_size = os.stat(self.test_image_path).st_size

        new_path = diet.squeeze(self.test_image_path)
        new_size = os.stat(new_path).st_size

        if self.have_jpeg_tools():
            self.assertEqual(new_path, self.test_image_path)
            self.assertTrue(new_size < old_size)
        else:
            print "Install jpegtran or jpegoptim to test shrinking"



from django.conf import settings

DIET_DEBUG = getattr(settings, 'DIET_DEBUG', False)

DIET_JPEGOPTIM = getattr(settings, 'DIET_JPEGOPTIM', True)
DIET_JPEGTRAN = getattr(settings, 'DIET_JPEGTRAN', True)
DIET_GIFSICLE = getattr(settings, 'DIET_GIFSICLE', True)
DIET_OPTIPNG = getattr(settings, 'DIET_OPTIPNG', True)
DIET_ADVPNG = getattr(settings, 'DIET_ADVPNG', True)
DIET_PNGCRUSH = getattr(settings, 'DIET_PNGCRUSH', True)
DIET_PNGQUANT = getattr(settings, 'DIET_PNGQUANT', True)
DIET_PNGQUANT_BRUTE = getattr(settings, 'DIET_PNGQUANT_BRUTE', False)
DIET_IMAGE_BACKUPS = getattr(settings, 'DIET_IMAGE_BACKUPS', True)
DIET_COMPRESS_STATIC_IMAGES = getattr(settings, 'DIET_COMPRESS_STATIC_IMAGES', False)
DIET_COMPRESSED_FLAG = getattr(settings, 'DIET_COMPRESSED_FLAG', 'image_diet_compressed')
DIET_FLAG_PROCESSED_FILE = getattr(settings, 'DIET_FLAG_PROCESSED_FILE', DIET_COMPRESSED_FLAG)
DIET_NEW_FILES_ONLY = getattr(settings, 'DIET_NEW_FILES_ONLY', True)



import os
from os.path import join
from django.core.management.base import BaseCommand
from image_diet.diet import squeeze


class Command(BaseCommand):
    args = '<dir1> [<dir2>...]'
    help = "Scan directories and subdirectories for images and compress them."

    def handle(self, *args, **options):
        for dirname in args:
            for (root, dirs, files) in os.walk(dirname):
                for filename in files:
                    filepath = join(root, filename)
                    squeeze(filepath)



from setuptools import setup, find_packages

long_description = '''\
django-image-diet is a Django application for removing unnecessary bytes from image
files.  It optimizes images without changing their look or visual quality
("losslessly").

It works on images in JPEG, GIF and PNG formats and will leave others
unchanged. Provides a seemless integration with easy_thumbnails app, but can
work with others too.'''

setup(
    author="Arabel.la",
    author_email='geeks@arabel.la',
    name='django-image-diet',
    version='0.1.2',
    description='Remove unnecessary bytes from images',
    long_description=long_description,
    url='https://github.com/ArabellaTech/django-image-diet',
    platforms=['OS Independent'],
    license='MIT License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Utilities',
    ],
    install_requires=[
        'django>=1.3',
    ],
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False
)



import os
from os.path import join, dirname
from shutil import copyfile
from django.test import TestCase
from image_diet.management.commands import diet_images

TEST_DIR = join(dirname(__file__), 'test_files')


class DietCommandTest(TestCase):
    def setUp(self):
        image_path = join(TEST_DIR, 'stockholm.jpg')

        self.nested_dir = join('dir1', 'dir2', 'dir3')
        self.test_root_dir = join(TEST_DIR, 'dir1')

        os.makedirs(join(TEST_DIR, self.nested_dir))

        self.test_image_path = join(TEST_DIR, self.nested_dir, 'stockholm.jpg')
        copyfile(image_path, self.test_image_path)

    def tearDown(self):
        os.remove(self.test_image_path)
        os.chdir(TEST_DIR)
        os.removedirs(self.nested_dir)

    def test_diet_images(self):
        old_size = os.stat(self.test_image_path).st_size
        action = diet_images.Command()
        action.handle(self.test_root_dir)
        new_size = os.stat(self.test_image_path).st_size

        self.assertTrue(new_size < old_size)



from django.dispatch import receiver
from image_diet.diet import squeeze

try:
    from easy_thumbnails.signals import saved_file, thumbnail_created

    @receiver(saved_file)
    def optimize_file(sender, fieldfile, **kwargs):
        squeeze(fieldfile.path)

    @receiver(thumbnail_created)
    def optimize_thumbnail(sender, **kwargs):
        squeeze(sender.path)
except ImportError:
    pass



import tempfile
import os
import time
import datetime
from optparse import make_option

from django.core.management.base import BaseCommand  # , CommandError
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from basic_cms import settings as app_settings
from image_diet import settings
from image_diet.diet import squeeze


class Command(BaseCommand):
    help = "Compress images on external filesystem using image_diet. Creates a backup copy of compressed files"

    option_list = BaseCommand.option_list + (
        make_option(
            "--new_only",
            action='store_true',
            default=False,
            dest='new_only',
            help='Compress only new images',
        ),
        make_option(
            "--directory",
            action='append',
            type='string',
            default=[],
            dest='directories',
            help='Directory to compress. Options: all, path, default all. Recursive.'
        ),
    )

    def handle(self, new_only, directories, **options):
        if 'image_diet' not in app_settings.INSTALLED_APPS:
            raise NotImplementedError("You need to install image_diet to use this command")
        timestamp = time.time()
        timestamp = datetime.datetime.fromtimestamp(timestamp).strftime('-%Y-%m-%d-%H:%M:%S')

        # todo:
        # add more options, like
        # directories - all, cms, directory list
        if not directories:
            directories.append('all')

        def process_file(path):
            """Process single file"""
            # failsafe copy of file
            copy = default_storage.open(path, 'rb')
            default_storage.save(path + timestamp, copy)
            copy.close()
            try:
                path = default_storage.path(path)
                squeeze(path, backup=False, flag_processed_file=False, new_only=False)
            except NotImplementedError:
                if path[-1:] != os.sep:
                    pf = default_storage.open(path, 'rwb')
                    image = pf.read()
                    tmpfilehandle, tmpfilepath = tempfile.mkstemp()
                    tmpfilehandle = os.fdopen(tmpfilehandle, 'wb')
                    tmpfilehandle.write(image)
                    tmpfilehandle.close()
                    squeeze(tmpfilepath)
                    tmpfilehandle = open(tmpfilepath)
                    pf.close()
                    default_storage.save(path, tmpfilehandle)
                    os.remove(tmpfilepath)

        def compress_files(directories, dirtree, new_only=False):
            compressed_flag = settings.DIET_FLAG_PROCESSED_FILE

            for f in directories[1]:  # files from listdir

                if f and f.lower().endswith(('.png', '.jpg', '.jpeg', '.gif')):  # sometimes if == [u'']
                    dir_path = os.sep.join(dirtree)
                    path = os.path.join(dir_path, f)
                    flagged_file_name = '.%s.%s' % (f, compressed_flag)
                    flag_path = os.path.join(dir_path, flagged_file_name)
                    print("Processing %s" % path)
                    if new_only:
                        should_process_file = False

                        if not default_storage.exists(flag_path):
                            should_process_file = True
                        else:
                            file_mt = default_storage.modified_time(path)
                            try:
                                flag_mt = default_storage.modified_time(flag_path)
                            except AttributeError:
                                flag_mt = 0
                            if flag_mt < file_mt:
                                should_process_file = True

                        if should_process_file:
                            process_file(path)
                    else:
                        process_file(path)

                    # add flag, for all files. This flag is used only when "new_only" option is called.
                    if default_storage.exists(flag_path):
                        default_storage.delete(flag_path)
                    default_storage.save(flag_path, ContentFile(""))

            for d in directories[0]:  # directories from list_dir
                dirtree.append(d)
                d = default_storage.listdir(os.sep.join(dirtree))
                compress_files(d, dirtree, new_only)
                dirtree.pop()  # remove last item, not needed anymore

        # proper start
        if 'all' in directories:
            directories = default_storage.listdir(os.path.sep)[0]

        for directory in directories:
            if directory:
                dirtree = [directory]
                d = default_storage.listdir(directory)
                compress_files(d, dirtree, new_only)
